@@ -31,11 +31,8 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 
-#ifdef HAVE_CLUTTER
-#include <clutter/clutter.h>
-#endif
-
 #include <libgames-support/games-debug.h>
+#include <libgames-support/games-glib-compat.h>
 #include <libgames-support/games-runtime.h>
 #include <libgames-support/games-string-utils.h>
 
@@ -46,11 +43,7 @@
 
 #define DELAYED_CALLBACK_DELAY (50)
 
-#if GLIB_CHECK_VERSION (2, 10, 0)
 #define I_(string) g_intern_static_string (string)
-#else
-#define I_(string) string
-#endif
 
 struct _AisleriotGame
 {
@@ -59,13 +52,14 @@ struct _AisleriotGame
   GPtrArray *slots;
 
   char *game_file;
-  guint timeout;
-  guint score;
   guint32 seed;
   guint delayed_call_timeout_id;
 
   GTimer *timer;
 
+  int timeout;
+  int score;
+
   double width;
   double height;
 
@@ -156,7 +150,7 @@ set_game_state (AisleriotGame *game,
 
 static void
 set_game_score (AisleriotGame *game,
-                guint score)
+                int score)
 {
   if (score != game->score) {
     game->score = score;
@@ -239,23 +233,10 @@ clear_slots (AisleriotGame *game,
   for (i = 0; i < n_slots; ++i) {
     ArSlot *slot = game->slots->pdata[i];
 
-#ifdef HAVE_CLUTTER
-    if (slot->slot_renderer) {
-      clutter_actor_destroy (slot->slot_renderer);
-      g_object_unref (slot->slot_renderer);
-    }
-
-    g_byte_array_free (slot->old_cards, TRUE);
-#else
     g_ptr_array_free (slot->card_images, TRUE);
-#endif /* HAVE_CLUTTER */
     g_byte_array_free (slot->cards, TRUE);
 
-#if GLIB_CHECK_VERSION (2, 10, 0)
     g_slice_free (ArSlot, slot);
-#else
-    g_free (slot);
-#endif
   }
 
   g_ptr_array_set_size (game->slots, 0);
@@ -630,11 +611,7 @@ cscmi_add_slot (SCM slot_data)
 #endif /* GNOME_ENABLE_DEBUG */
 
   /* create and initialize slot */
-#if GLIB_CHECK_VERSION (2, 10, 0)
   slot = g_slice_new0 (ArSlot);
-#else
-  slot = g_new0 (Slot, 1);
-#endif
 
   g_ptr_array_add (game->slots, slot);
 
@@ -652,11 +629,7 @@ cscmi_add_slot (SCM slot_data)
   slot->expanded_down = expanded_down != FALSE;
   slot->expanded_right = expanded_right != FALSE;
 
-#ifdef HAVE_CLUTTER
-  slot->old_cards = g_byte_array_sized_new (SLOT_CARDS_N_PREALLOC);
-#else
   slot->card_images = g_ptr_array_sized_new (SLOT_CARDS_N_PREALLOC);
-#endif
 
   slot->needs_update = TRUE;
 
@@ -930,7 +903,7 @@ scm_get_score (void)
 {
   AisleriotGame *game = app_game;
 
-  return scm_from_uint (game->score);
+  return scm_from_int (game->score);
 }
 
 static SCM
@@ -946,11 +919,11 @@ static SCM
 scm_add_to_score (SCM delta)
 {
   AisleriotGame *game = app_game;
-  guint new_score;
+  int new_score;
 
   new_score = game->score + scm_num2int (delta, SCM_ARG1, NULL);
   set_game_score (game, new_score);
-  return scm_from_uint (new_score);
+  return scm_from_int (new_score);
 }
 
 static SCM
@@ -972,7 +945,7 @@ scm_get_timeout (void)
 
   g_warning ("(get-timeout) unimplemented\n");
 
-  return scm_from_uint (game->timeout);
+  return scm_from_int (game->timeout);
 }
 
 static void
@@ -1257,7 +1230,7 @@ aisleriot_game_get_property (GObject *object,
       g_value_set_string (value, game->game_file);
       break;
     case PROP_SCORE:
-      g_value_set_uint (value, game->score);
+      g_value_set_int (value, game->score);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -1341,42 +1314,48 @@ aisleriot_game_class_init (AisleriotGameClass *klass)
      PROP_CAN_UNDO,
      g_param_spec_boolean ("can-undo", NULL, NULL,
                            FALSE,
-                           G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+                           G_PARAM_READABLE |
+                           G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property
     (gobject_class,
      PROP_CAN_REDO,
      g_param_spec_boolean ("can-redo", NULL, NULL,
                            FALSE,
-                           G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+                           G_PARAM_READABLE |
+                           G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property
     (gobject_class,
      PROP_CAN_DEAL,
      g_param_spec_boolean ("can-deal", NULL, NULL,
                            FALSE,
-                           G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+                           G_PARAM_READABLE |
+                           G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property
     (gobject_class,
      PROP_GAME_FILE,
      g_param_spec_string ("game-file", NULL, NULL,
                           NULL,
-                          G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+                          G_PARAM_READABLE |
+                          G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property
     (gobject_class,
      PROP_SCORE,
-     g_param_spec_uint ("score", "", "",
-                        0, G_MAXUINT, 0,
-                        G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+     g_param_spec_int ("score", NULL, NULL,
+                       G_MININT, G_MAXINT, 0,
+                       G_PARAM_READABLE |
+                       G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property
     (gobject_class,
      PROP_STATE,
-     g_param_spec_uint ("state", "", "",
+     g_param_spec_uint ("state", NULL, NULL,
                         0, LAST_GAME_STATE, 0, 
-                        G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+                        G_PARAM_READABLE |
+                        G_PARAM_STATIC_STRINGS));
 
   /* Initialise our scheme interfaces */
   cscm_init ();
@@ -1384,6 +1363,112 @@ aisleriot_game_class_init (AisleriotGameClass *klass)
 
 /* public API */
 
+/**
+ * ar_slot_get_slot_type:
+ * @slot: a #ArSlot
+ *
+ * Returns: the slot type of @slot
+ */
+ArSlotType
+ar_slot_get_slot_type (ArSlot *slot)
+{
+  g_return_val_if_fail (slot != NULL, AR_SLOT_UNKNOWN);
+
+  return slot->type;
+}
+
+/**
+ * ar_slot_get_type_string:
+ * @slot: a #ArSlot
+ *
+ * Returns: a string describing the slot type
+ */
+const char *
+ar_slot_get_type_string (ArSlot *slot)
+{
+  const char *text = NULL;
+
+  g_return_val_if_fail (slot != NULL, NULL);
+
+  switch (slot->type) {
+    case AR_SLOT_UNKNOWN:
+      text = NULL;
+      break;
+    case AR_SLOT_FOUNDATION:
+      /* Translators: this is the name of a type of card slot */
+      text = C_("slot type", "foundation");
+      break;
+    case AR_SLOT_RESERVE:
+      /* Translators: this is the name of a type of card slot */
+      text = C_("slot type", "reserve");
+      break;
+    case AR_SLOT_STOCK:
+      /* Translators: this is the name of a type of card slot */
+      text = C_("slot type", "stock");
+      break;
+    case AR_SLOT_TABLEAU:
+      /* Translators: this is the name of a type of card slot */
+      text = C_("slot type", "tableau");
+      break;
+    case AR_SLOT_WASTE:
+      /* Translators: this is the name of a type of card slot */
+      text = C_("slot type", "waste");
+      break;
+  }
+
+  return text;
+}
+
+/**
+ * ar_slot_get_hint_string:
+ * @slot: a #ArSlot
+ *
+ * Returns: a string describing the slot type
+ */
+char *
+ar_slot_get_hint_string (ArSlot *slot,
+                         int cardid)
+{
+  const char *card_name;
+
+  g_return_val_if_fail (slot != NULL, NULL);
+
+  if (slot->type == AR_SLOT_UNKNOWN)
+    return NULL;
+
+  if (cardid < 0)
+    return g_strdup (ar_slot_get_type_string (slot));
+
+  card_name = ar_card_get_locale_name (CARD (slot->cards->data[cardid]));
+
+  switch (slot->type) {
+    case AR_SLOT_FOUNDATION:
+      /* Translators: %s is the name of the card; "foundation" is the name of a type of card slot */
+      return g_strdup_printf (C_("slot hint", "%s on foundation"), card_name);
+
+    case AR_SLOT_RESERVE:
+      /* Translators: %s is the name of the card; "reserve" is the name of a type of card slot */
+      return g_strdup_printf (C_("slot hint", "%s on reserve"), card_name);
+
+    case AR_SLOT_STOCK:
+      /* Translators: %s is the name of the card; "stock" is the name of a type of card slot */
+      return g_strdup_printf (C_("slot hint", "%s on stock"), card_name);
+
+    case AR_SLOT_TABLEAU:
+      /* Translators: %s is the name of the card; "tableau" is the name of a type of card slot */
+      return g_strdup_printf (C_("slot hint", "%s on tableau"), card_name);
+
+    case AR_SLOT_WASTE:
+      /* Translators: %s is the name of the card; "waste" is the name of a type of card slot */
+      return g_strdup_printf (C_("slot hint", "%s on waste"), card_name);
+
+    default:
+      g_assert_not_reached ();
+  }
+
+  return NULL;
+}
+
 /**
  * aisleriot_game_error_quark:
  *
@@ -2084,11 +2169,7 @@ aisleriot_game_option_free (AisleriotGameOption *option)
   g_return_if_fail (option != NULL);
 
   g_free (option->display_name);
-#if GLIB_CHECK_VERSION (2, 10, 0)
   g_slice_free (AisleriotGameOption, option);
-#else
-  g_free (option);
-#endif
 }
 
 /**
@@ -2145,12 +2226,7 @@ aisleriot_game_get_options (AisleriotGame *game)
 
       entrystate = SCM_NFALSEP (scm_list_ref (entry, scm_from_uint (1)));
 
-#if GLIB_CHECK_VERSION (2, 10, 0)
       option = g_slice_new (AisleriotGameOption);
-#else
-      option = g_new (AisleriotGameOption, 1);
-#endif
-
       option->display_name = g_strdup (entrynamestr);
       option->type = type;
       option->value = bit;
@@ -2390,41 +2466,3 @@ aisleriot_game_deal_cards (AisleriotGame *game)
   aisleriot_game_end_move (game);
   aisleriot_game_test_end_of_game (game);
 }
-
-#ifdef HAVE_CLUTTER
-
-void
-aisleriot_game_get_card_offset (ArSlot *slot,
-                                guint card_num,
-                                gboolean old_cards,
-                                gint *xoff, gint *yoff)
-{
-  gint n_cards, exposed;
-
-  if (old_cards) {
-    n_cards = (gint) slot->old_cards->len;
-    exposed = (gint) slot->old_exposed;
-  } else {
-    n_cards = (gint) slot->cards->len;
-    exposed = (gint) slot->exposed;
-  }
-
-  if (card_num >= n_cards - exposed) {
-    gint idx = card_num + exposed - n_cards;
-    *xoff = slot->pixeldx * idx;
-    *yoff = slot->pixeldy * idx;
-  } else {
-    *xoff = 0;
-    *yoff = 0;
-  }
-}
-
-void
-aisleriot_game_reset_old_cards (ArSlot *slot)
-{
-  g_byte_array_set_size (slot->old_cards, 0);
-  g_byte_array_append (slot->old_cards, slot->cards->data, slot->cards->len);
-  slot->old_exposed = slot->exposed;
-}
-
-#endif /* HAVE_CLUTTER */
